/*
 Library for BMP085 Sensor
 */


#include "Arduino.h"
#include "Wire.h"
#include "BMPLib.h"

BMPLib::BMPLib()		// Contructor
{
}


bool BMPLib::begin(void)	// Calibration function
{

	if (bmp085Read(0xD0) != 0x55) return false;

	OSS = 0;  // Oversampling Setting

	// Calibration values
	ac1 = bmp085ReadInt(0xAA);
	ac2 = bmp085ReadInt(0xAC); 
	ac3 = bmp085ReadInt(0xAE); 
	ac4 = bmp085ReadInt(0xB0);
	ac5 = bmp085ReadInt(0xB2);
	ac6 = bmp085ReadInt(0xB4);
	b1 = bmp085ReadInt(0xB6); 
	b2 = bmp085ReadInt(0xB8);
	mb = bmp085ReadInt(0xBA);
	mc = bmp085ReadInt(0xBC);
	md = bmp085ReadInt(0xBE);

	// b5 is calculated in bmp085GetTemperature(...), this variable is also used in bmp085GetPressure(...)
	// so ...Temperature(...) must be called before ...Pressure(...).
	b5 = 0;		// for now 


	return true;
}

void BMPLib::calibrate(void)	// Calibration function
{
	int p0 = 0;
	for(int z=0; z<20; z++){
		read_values();
		p0 = p0 + pressure;
	}
	p0 = p0/20;
}	

void BMPLib::read_values(void){	
	last_temperature = temperature;
	last_pressure = pressure;
	last_altitude_meters = altitude_meters;
	last_altitude_feet = altitude_feet;
	
	temperature = bmp085GetTemperature(bmp085ReadUT())*0.1;    //reads temperature from bmp085
	//temperature1 = temperature * 0.1;                      //converts decimal point of temperature
	pressure = bmp085GetPressure(bmp085ReadUP());          //reads pressure from bmp085
	altitude_meters = (float)44330 * (1 - pow(((float) pressure/p0), 0.190295));    //calculates new altitude
	altitude_feet = (altitude_meters)*(3.2808399);               //converts altitude from meters to feet
}

short BMPLib::bmp085GetTemperature(unsigned int ut)		// Calculate Temperature given ut.  Value returend will be in unites of 0.1 deg C
{
  long x1, x2;
  
  x1 = (((long)ut - (long)ac6)*(long)ac5) >> 15;
  x2 = ((long)mc << 11)/(x1 + md);
  b5 = x1 + x2;

  return ((b5 + 8)>>4);  
}


long BMPLib::bmp085GetPressure(unsigned long up)		// Calculate pressure given up.  Calibration values must be know, b5 is reguired (call bmp085GetTemperature(...) first). Value returned in units of Pa.
{
  long x1, x2, x3, b3, b6, p;
  unsigned long b4, b7;
  
  b6 = b5 - 4000;
  // Calculate B3
  x1 = (b2 * (b6 * b6)>>12)>>11;
  x2 = (ac2 * b6)>>11;
  x3 = x1 + x2;
  b3 = (((((long)ac1)*4 + x3)<<OSS) + 2)>>2;
  
  // Calculate B4
  x1 = (ac3 * b6)>>13;
  x2 = (b1 * ((b6 * b6)>>12))>>16;
  x3 = ((x1 + x2) + 2)>>2;
  b4 = (ac4 * (unsigned long)(x3 + 32768))>>15;
  
  b7 = ((unsigned long)(up - b3) * (50000>>OSS));
  if (b7 < 0x80000000)
    p = (b7<<1)/b4;
  else
    p = (b7/b4)<<1;
    
  x1 = (p>>8) * (p>>8);
  x1 = (x1 * 3038)>>16;
  x2 = (-7357 * p)>>16;
  p += (x1 + x2 + 3791)>>4;
  
  return p;
}



char BMPLib::bmp085Read(unsigned char address)		// Read 1 byte from the BMP085 at 'address'
{
  unsigned char data;
  
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();
  
  Wire.requestFrom(BMP085_ADDRESS, 1);
  while(!Wire.available());
    
  return Wire.read();
}


int BMPLib::bmp085ReadInt(unsigned char address)		// Read 2 bytes from the BMP085, First byte from 'address', second byte from 'address'+1
{
  unsigned char msb, lsb;
  
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();
  
  Wire.requestFrom(BMP085_ADDRESS, 2);
  while(Wire.available()<2);
  msb = Wire.read();
  lsb = Wire.read();
  
  return (int) msb<<8 | lsb;
}


unsigned int BMPLib::bmp085ReadUT()		// Read the uncompensated temperature value
{
  unsigned int ut;
  
  // Write 0x2E into Register 0xF4
  // This requests a temperature reading
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF4);
  Wire.write(0x2E);
  Wire.endTransmission();
  
  // Wait at least 4.5ms
  delay(5);
  
  // Read two bytes from registers 0xF6 and 0xF7
  ut = bmp085ReadInt(0xF6);
  return ut;
}


unsigned long BMPLib::bmp085ReadUP()		// Read the uncompensated pressure value
{
  unsigned char msb, lsb, xlsb;
  unsigned long up = 0;
  
  // Write 0x34+(OSS<<6) into register 0xF4
  // Request a pressure reading w/ oversampling setting
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF4);
  Wire.write(0x34 + (OSS<<6));
  Wire.endTransmission();
  
  // Wait for conversion, delay time dependent on OSS
  delay(2 + (3<<OSS));
  
  // Read register 0xF6 (MSB), 0xF7 (LSB), and 0xF8 (XLSB)
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF6);
  Wire.endTransmission();
  Wire.requestFrom(BMP085_ADDRESS, 3);
  
  // Wait for data to become available
  while(Wire.available() < 3);
  msb = Wire.read();
  lsb = Wire.read();
  xlsb = Wire.read();
  
  up = (((unsigned long) msb << 16) | ((unsigned long) lsb << 8) | (unsigned long) xlsb) >> (8-OSS);
  
  return up;
}




